
class Timer
{
  public:
    bool hasExpired();
    void start(int timeout);
    Timer();
  private:
    unsigned long nextTimeout;
};

class NumberDisplay
{
  public:
    void increment();
    int getValue();
    NumberDisplay(int rollOverLimit);
    String getDisplayValue();
  private:
    int value;
    int limit;
};

class ClockDisplay
{
  public:
    String getTime();
    ClockDisplay();
    void timeTick();
    void setTime(int selected);
  private:
    NumberDisplay hours;
    NumberDisplay minutes;
    NumberDisplay seconds;
};


const int S_RUN_CLOCK = 0;
const int S_SELECT_SECONDS = 1;
const int S_SELECT_MINUTES = 2;
const int S_SELECT_HOURS = 3;
const int light = 10 ;

const int SELECT_BTN = 7;
const int SET_BTN = 6;

long uren = 0;
long urenTemp = 0;

int currentState = S_RUN_CLOCK;

#include <LiquidCrystal.h>


LiquidCrystal lcd(12, 11, 5, 4, 3, 2);
ClockDisplay myClock;

Timer timer1;

void setup() {
  pinMode(SELECT_BTN, INPUT); 
  pinMode(SET_BTN, INPUT);
  lcd.begin(16, 2);
  
}

void loop() {
  String timeString = myClock.getTime();

  lcd.setCursor(0, 0);
  lcd.print(timeString);
  
 /*
  if(uren != urenTemp) {
  	digitalWrite(10, HIGH);
    urenTemp = uren;
    delay(500);
    digitalWrite(10, LOW);
  }
  */
  
  switch (currentState) {
    case S_RUN_CLOCK:
      if (isButtonPressed(SELECT_BTN)) {
        currentState = S_SELECT_SECONDS;
        printSelect(currentState);
      }
      if(timer1.hasExpired())
  	  {
    	timer1.start(1000);
    	myClock.timeTick();
  	  }
      break;
  

    case S_SELECT_SECONDS:
      if (isButtonPressed(SELECT_BTN)) {
        currentState = S_SELECT_MINUTES;
        printSelect(currentState);
      }
      if(isButtonPressed(SET_BTN))
      {
        myClock.setTime(currentState);
      }
    break;
    case S_SELECT_MINUTES:
      if (isButtonPressed(SELECT_BTN)) {
        currentState = S_SELECT_HOURS;
        printSelect(currentState);
      }
      if(isButtonPressed(SET_BTN))
      {
        myClock.setTime(currentState);
      }
    break;
    case S_SELECT_HOURS:
      if (isButtonPressed(SELECT_BTN)) {
        currentState = S_RUN_CLOCK;
        printSelect(currentState);
      }
      if(isButtonPressed(SET_BTN))
      {
        myClock.setTime(currentState);
      }
    break;
  }

  delay(10);
}

bool isButtonPressed(int pin)
{
  if (digitalRead(pin) == HIGH)
  {
    while(digitalRead(pin)==HIGH){}
    return true;
  }
  else
  {
    return false;
  }
}
void printSelect(int currentState) {
  lcd.setCursor(0, 1);
  lcd.print("        ");
  if (currentState != 0) {

    lcd.setCursor(3 * (3 - currentState), 1);
    lcd.print("^^");
  }
}	
const int SECONDS = 1;
const int MINUTES = 2;
const int HOURS = 3;



void ClockDisplay::timeTick()
{
  seconds.increment();
  if (seconds.getValue() == 0) {
    minutes.increment();
    if (minutes.getValue() == 0) {
      hours.increment();
    }
  }
}


String ClockDisplay::getTime()
{

  String timeString = hours.getDisplayValue() + ":" + minutes.getDisplayValue() + ":" + seconds.getDisplayValue();
  return timeString;
}


void ClockDisplay::setTime(int selected)
{
  switch(selected)
  {
    case SECONDS:
    seconds.increment();
    break;
    case MINUTES:
    minutes.increment();
    break;
    case HOURS:
    hours.increment();
    break;
  }
}



ClockDisplay::ClockDisplay(): hours(24), minutes(60), seconds(60)
{
}


void NumberDisplay::increment()
{
  value += 1;
  value %= limit;
}


int NumberDisplay::getValue()
{
  return value;
}


String NumberDisplay::getDisplayValue()
{
  if (value < 10)
  {
    return "0" + String(value);
  }
  else
  {
    return String(value);
  }
}


NumberDisplay::NumberDisplay(int rollOverLimit): value(0)
{
  limit = rollOverLimit;
}

bool Timer::hasExpired()
{
  return (millis()>nextTimeout);
}

void Timer::start(int timeout)
{
  nextTimeout = millis() + timeout;
}

Timer::Timer() : nextTimeout(0) {};
